package com.kps.androbuy;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.message.BasicNameValuePair;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.text.Html;
import android.text.method.LinkMovementMethod;
import android.util.Log;
import android.util.Xml.Encoding;
import android.view.Menu;
import android.view.View;
import android.view.Window;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesClient.ConnectionCallbacks;
import com.google.android.gms.common.GooglePlayServicesClient.OnConnectionFailedListener;
import com.google.android.gms.location.LocationClient;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.kps.androbuy.data.Entry;
import com.squareup.picasso.Picasso;

public class MainActivity extends ActionBarActivity implements
		ConnectionCallbacks, OnConnectionFailedListener, LocationListener {
	private LocationClient mLocationClient = null;
	private String ebayRequete = null;
	public static final String RESULTS = "results";

	// These settings are the same as the settings for the map. They will in
	// fact give you updates
	// at the maximal rates currently possible.
	private static final LocationRequest REQUEST = LocationRequest.create()
			.setPriority(LocationRequest.PRIORITY_LOW_POWER);

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		if (!requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS))
			Log.d("Progress", "Can't progress");

		// Set up and launch the location client
		setUpLocationClientIfNeeded();
		connectLocationClientIfNeeded();

		// Set the content
		setContentView(R.layout.activity_main);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);

		return true;
	}

	@Override
	public void onConnectionFailed(ConnectionResult result) {
		Toast.makeText(this,
				getResources().getString(R.string.error_no_location),
				Toast.LENGTH_SHORT).show();
	}

	@Override
	public void onConnected(Bundle connectionHint) {
		mLocationClient.requestLocationUpdates(REQUEST, this);
	}

	@Override
	public void onDisconnected() {
		// Nothing to do, so... Procrastination
	}

	@Override
	public void onLocationChanged(Location location) {
		// OK
	}

	/**
	 * Réponse à un clic sur "Rechercher"
	 * 
	 * @param v
	 */
	public void productSearchClick(View v) {
		String query = ((EditText) findViewById(R.id.main_search_field))
				.getText().toString();
		if (query.length() > 1) {
			setProgressBarIndeterminate(true);
			setProgressBarIndeterminateVisibility(true);
			searchProduct(query);
		} else
			Toast.makeText(this,
					getResources().getString(R.string.error_no_input),
					Toast.LENGTH_SHORT).show();
	}

	/**
	 * Recherche d'un produit d'après une requête
	 * 
	 * @param query
	 */
	private void searchProduct(String query) {
		new ProductQueryTask().execute(query);
	}

	/**
	 * Ajout d'un produit à l'URL classique de recherche
	 * 
	 * @param product
	 * @return
	 */
	private String addProductToQuery(String product) {
		String url = "http://svcs.ebay.com/services/search/FindingService/v1";

		if (!url.endsWith("?"))
			url += "?";

		List<NameValuePair> params = new LinkedList<NameValuePair>();
		params.add(new BasicNameValuePair("SECURITY-APPNAME",
				"VincentG-9861-4c81-8ae8-f48b1e390190"));
		params.add(new BasicNameValuePair("GLOBAL-ID", "EBAY-FR"));
		params.add(new BasicNameValuePair("OPERATION-NAME",
				"findItemsByKeywords"));
		params.add(new BasicNameValuePair("SERVICE-VERSION", "1.0.0"));
		params.add(new BasicNameValuePair("REST-PAYLOAD", "TRUE"));
		params.add(new BasicNameValuePair("RESPONSE-DATA-FORMAT", "XML"));
		params.add(new BasicNameValuePair("keywords", product));
		params.add(new BasicNameValuePair("paginationInput.entriesPerPage",
				"25"));
		url += URLEncodedUtils.format(params, Encoding.UTF_8.toString());

		Log.d("AddProduct", url);
		return url;
	}

	/**
	 * Ajout d'un produit et d'une localisation à l'URL classique de recherche
	 * 
	 * @param product
	 * @param location
	 * @return
	 */
	private String addProductToQuery(String product, Location location) {
		String url = addProductToQuery(product);
		Geocoder geoCoder = new Geocoder(this, Locale.getDefault());
		List<Address> addresses = null;
		Address address = null;

		try {
			if (location != null)
				addresses = geoCoder.getFromLocation(location.getLatitude(),
						location.getLongitude(), 1);
			if (addresses != null && addresses.size() > 0)
				address = addresses.get(0);
		} catch (IOException e) {
			e.printStackTrace();
		}

		if (address != null) {
			List<NameValuePair> params = new LinkedList<NameValuePair>();
			params.add(new BasicNameValuePair("buyerPostalCode", address
					.getPostalCode()));
			params.add(new BasicNameValuePair("MaxDistance", "100"));
			params.add(new BasicNameValuePair("sortOrder", "DistanceNearest"));

			url += "&"
					+ URLEncodedUtils.format(params, Encoding.UTF_8.toString());
		}

		Log.d("AddProduct", url);
		return url;
	}

	/**
	 * Parsing des résultats d'une recherche
	 * 
	 * @param parser
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	private ArrayList<Entry> parseXML(XmlPullParser parser)
			throws XmlPullParserException, IOException {
		ArrayList<Entry> products = null;
		int eventType = parser.getEventType();
		Entry currentProduct = null;

		while (eventType != XmlPullParser.END_DOCUMENT) {
			String name = null;
			switch (eventType) {
			case XmlPullParser.START_TAG:
				name = parser.getName();

				if (name.compareTo("searchResult") == 0) {
					products = new ArrayList<Entry>();
				}
				if (name.compareTo("item") == 0) {
					currentProduct = new Entry();
				} else if (currentProduct != null) {
					if (name.compareTo("title") == 0) {
						currentProduct.setTitle(parser.nextText());
					} else if (name.compareTo("galleryURL") == 0) {
						currentProduct.setImageURL(parser.nextText());
					} else if (name.compareTo("viewItemURL") == 0) {
						currentProduct.setLinkURL(parser.nextText());
					} else if (name.compareTo("convertedCurrentPrice") == 0) {
						currentProduct.setPrice(Double.parseDouble(parser
								.nextText()));
					}
				}
				break;
			case XmlPullParser.END_TAG:
				name = parser.getName();
				if (name.compareTo("item") == 0 && currentProduct != null) {
					products.add(currentProduct);
					currentProduct = null;
				}
			}
			eventType = parser.next();
		}
		// printProducts(products);
		return products;
	}

	/**
	 * Impression des produits d'une recherche
	 * 
	 * @param products
	 */
	@SuppressWarnings("unused")
	private void printProducts(ArrayList<Entry> products) {
		String content = "";
		Iterator<Entry> it = products.iterator();
		while (it.hasNext()) {
			Entry currProduct = it.next();
			content += "Product :" + currProduct.getTitle() + "\n";
			content += " url :" + currProduct.getLinkURL() + "\n";
			content += " img :" + currProduct.getImageURL() + "\n";

			Log.d("product", content);
			content = "";
		}

	}

	/**
	 * Set this.mLocationClient if needed
	 */
	private void setUpLocationClientIfNeeded() {
		if (this.mLocationClient == null) {
			this.mLocationClient = new LocationClient(getApplicationContext(),
					this, // ConnectionCallbacks
					this); // OnConnectionFailedListener
		}
	}

	/**
	 * Connect the locationClient if we need to
	 */
	private void connectLocationClientIfNeeded() {
		if (!this.mLocationClient.isConnected()
				&& !this.mLocationClient.isConnecting())
			this.mLocationClient.connect();
	}

	private class ProductQueryTask extends
			AsyncTask<String, Void, ArrayList<Entry>> {

		@Override
		protected ArrayList<Entry> doInBackground(String... params) {
			XmlPullParserFactory pullParserFactory;
			ArrayList<Entry> results = null;

			try {
				pullParserFactory = XmlPullParserFactory.newInstance();
				XmlPullParser parser = pullParserFactory.newPullParser();

				if (mLocationClient.isConnected())
					ebayRequete = addProductToQuery(params[0],
							mLocationClient.getLastLocation());
				else
					ebayRequete = addProductToQuery(params[0]);
				URL urlEbayRequete = new URL(ebayRequete);

				URLConnection urlConnection = urlEbayRequete.openConnection();
				HttpURLConnection httpUrlConnection = (HttpURLConnection) urlConnection;
				// InputStream stream = httpUrlConnection.getInputStream();

				InputStream in_s = httpUrlConnection.getInputStream();
				parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES,
						false);
				parser.setInput(in_s, null);

				results = parseXML(parser);

			} catch (XmlPullParserException e) {

				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}

			return results;
		}

		@Override
		protected void onPostExecute(ArrayList<Entry> result) {
			super.onPostExecute(result);

			ImageView img = null;
			LinearLayout images = (LinearLayout) findViewById(R.id.main_images);
			TextView output = (TextView) findViewById(R.id.main_textView_description);
			output.setText(null);
			if (result != null)
				for (Entry entry : result) {
					output.append("<a href='" + entry.getLinkURL() + "'>"
							+ entry.getTitle() + "</a><br />Prix : $"
							+ entry.getPrice() + "<br /><br />");
					img = new ImageView(getApplicationContext());
					Picasso.with(getApplicationContext())
							.load(entry.getImageURL())
							.error(getResources().getDrawable(
									android.R.drawable.ic_dialog_alert))
							.resize(140, 140).into(img);
					images.addView(img);
				}
			output.setText(Html.fromHtml(output.getText().toString()));
			output.setMovementMethod(LinkMovementMethod.getInstance());
			setProgressBarIndeterminateVisibility(false);
		}
	}
}
